#version 3.7;

#include "colors.inc"     
#include "functions.inc"
#include "transforms.inc"
#include "textures.inc"
#include "woods.inc"

#declare SW_PHOT=on;
#declare SW_Grass=on;
#declare SW_Palms=on;

global_settings{
 assumed_gamma 1.0
 ambient_light 0.0 
 radiosity{ 
  // modified Outdoor settings from rad_def.inc
  #if (LoadRad=1)
    // load settings  
    pretrace_start 1
    pretrace_end 1 
    error_bound 0.1    // it's important to specify the same value used to save data
    recursion_limit 2  // also the same if specified
    always_sample off  // no more samples: specially useful with focal blur
  #else
    pretrace_start 0.08
    pretrace_end   0.01
    count 250
    nearest_count 20
    error_bound 0.1
    recursion_limit 2
    low_error_factor .9
    gray_threshold 0
    minimum_reuse 0.015
    brightness 0.3
    adc_bailout 0.01/2
  #end
 }
 
   #if (SW_PHOT)
      photons {
         spacing 0.00175
         max_trace_level 256
         autostop 0
         load_file "ZanzibarBeach.photons"
      }
   #end 
 
}

camera {
  location  <0,1,-5>
  up y
  right     x*image_width/image_height
  look_at   <0, 0, 0>  
}


sky_sphere {
   pigment {
      gradient y
      color_map {
         [ 0,0.25  color srgb < 0.392, 0.545, 0.706 >*1.1 
                   color srgb < 0.173, 0.325, 0.635 > ]
      }
      scale 2
      translate y*-0.01
   }
}

light_source { <1000,2000,-50>, White
   #if (SW_PHOT)
     photons { refraction on reflection on }
   #end

} 


/**************************************************************************************************
   Beach, height_field by Roman Reiner (IRTC February 2006, Firelake) with simplified transforms ;)
   http://ftp.uni-erlangen.de/pub/graphics/irtc/stills/2006-02-28/firelake.zip
***************************************************************************************************/

#declare ColourSeabottom=colour rgb < 0.75, 0.75, 0.30>; // used later by the sea

#declare beach_sand_tex=
  texture {
    pigment {
      gradient y
      scale 10
      translate <0,-4.75,0>
      pigment_map {
        [ 0 color ColourSeabottom ]
        [ 0.3 color ColourSeabottom ]
        [ 0.4 color <0.95,0.92,0.7>*0.75 ]
        [ .7 color <0.95,0.92,0.7>*0.95 ]
        [ .8 color <0.95,0.92,0.7> ]
        [ 1 color <0.95,0.92,0.7> ]
      }
    }
    finish {
      ambient 0
      diffuse .8
      roughness .5
      specular .01
    }
    normal {   
      granite -.05 poly_wave 2 scale .05/20 
    }
}


#local hf_trans = transform {
  translate <-0.60, -0.475, -0.333>
  scale <-250, 10, 150>
  rotate y*120   
  translate <0,0,10>
}  


#declare Beach = height_field {
  png "hf1map_mjf"
  smooth     
  transform { hf_trans }
    
  texture { beach_sand_tex }
}             

object { Beach
   #if (SW_PHOT)
      photons { collect off }
   #end
}

/**************************************************************************************************
  sea and sea bottom
   
  Textures based on Sean Day Wreck (IRTC June 2005, http://ftp.uni-erlangen.de/pub/graphics/irtc/stills/2005-06-30/wreck.zip)
  
  Isosurface from Jaime Vives Piqueres (http://www.ignorancia.org/index.php/galleries/old-images/sea-buoy//
  
  The sea and sea bottom were divided into a northern and a southern part. Photons were collected for the
  southern part only. The texture of the northern part had to be darkend a little bit.
***************************************************************************************************/

#declare SeaSize=1000;
#declare r_water = seed(8034);
#declare Water_Height =1;

#declare ColourSeabottom=colour rgb < 0.75, 0.75, 0.30>;

#declare SeabottomNorth=
   box{<-SeaSize*.6,-Water_Height,-5>,<SeaSize*.6,-0.001,SeaSize>
   texture { pigment { ColourSeabottom } }
   #if (SW_PHOT)
      photons { collect off } 
   #end
}
object { SeabottomNorth }


#declare SeabottomSouth=
   box{<-10,-Water_Height,-5>,<10,0,5>
   texture { pigment { ColourSeabottom } }
   #if (SW_PHOT)
      photons { collect on } 
   #end
}
object { SeabottomSouth }

#declare Coloursea=rgb <62,64,63>/64;

#declare seaTex = texture
{
        pigment{rgbf <Coloursea.red,Coloursea.green,Coloursea.blue,0.925>}
        //pigment{rgbft <COLoursea.red,COLoursea.green,COLoursea.blue,0.925,0.99>}
//        pigment { rgbt <1,1,1,1> }
        finish 
        {
                phong 1//4
                phong_size 100
                specular 1//2
                roughness 0.0005
                ambient 0
                diffuse 0.25      
                reflection {0.2,1 fresnel on}
                conserve_energy
        }
}

#declare darker=0.9;
#declare seaTexNorth = texture
{
        pigment{rgbf <darker*Coloursea.red,darker*Coloursea.green,darker*Coloursea.blue,0.925>}
        //pigment{rgbft <COLoursea.red,COLoursea.green,COLoursea.blue,0.925,0.99>}
//        pigment { rgbt <1,1,1,1> }
        finish 
        {
                phong 1//4
                phong_size 100
                specular 1//2
                roughness 0.0005
                ambient 0
                diffuse 0.25      
                reflection {0.2,1 fresnel on}
                conserve_energy
        }
}

#declare bubbleTex = texture 
{ 
        seaTex 
}       
texture
{
        pigment 
        { 
                bozo   
                scallop_wave
                turbulence 0.395
                octaves 9
                omega 0.675
                lambda 3.25
                pigment_map
                {
                        [ 0.0 rgbt <1, 1, 1, 1.0> ]  
                        [ 0.89 rgbt <1, 1, 1, 1.0> ]  
                        [ 0.98 rgbt <1, 1, 1, 0.7> ]  
                        [ 0.9995 rgbt <1, 1, 1, 0.005> ]  
                        [ 1.0 rgbt <1, 1, 1, 0> ]  
                        
                }
                scale <1.0, 0.3, 0.6>
                
        }
        finish { phong 1 phong_size 70 specular 1 roughness 0.01 ambient 0 diffuse 0.6 brilliance 0.6 }
}


// water surface functions
#declare f_water_waves1=
function{
 pattern{
  pigment_pattern{
   granite scale 4
   turbulence .1
   poly_wave .5
   color_map{[0 rgb 0][1 rgb -1]}
   scale 2
  }
 }
}
#declare f_water_waves2=
function{
 pattern{
  pigment_pattern{
   ripples
   turbulence .33
   color_map{[0 rgb 0][1 rgb 1]}
   scale 1
   rotate 360*rand(r_water)
  }
 }
}
#declare f_water=
function{
 f_water_waves1(x,0,z)*f_water_waves1(x,1,z)
}


#declare Water_SeanNorth=material {
   texture {
      gradient y 
      omega 0.585
      turbulence 0.0125
      texture_map {
         [ 0.0 seaTexNorth ]
         [ 0.54 seaTexNorth ]
         [ 0.554 bubbleTex ]
         [ 0.585 bubbleTex ]
         [ 0.6 seaTexNorth ]
         [ 1.0 seaTexNorth ]
      }
         
   }                       
   interior {
      ior 1.34
      dispersion 1.01    
      fade_distance 150
      fade_power 1001
      fade_color <0.0, 0.1, 0.075, 0.5>
      media {
         absorption (1-Coloursea)*0.1
      }     
   }
      
   scale <1,0.34,1>
}     

#declare Water_SeanSouth=material {
   texture {
      gradient y 
      omega 0.585
      turbulence 0.0125
      texture_map {
         [ 0.0 seaTex ]
         [ 0.54 seaTex ]
         [ 0.554 bubbleTex ]
         [ 0.585 bubbleTex ]
         [ 0.6 seaTex ]
         [ 1.0 seaTex ]
      }
         
   }                       
   interior {
      ior 1.34
      dispersion 1.01    
      fade_distance 150
      fade_power 1001
      fade_color <0.0, 0.1, 0.075, 0.5>
      media {
         absorption (1-Coloursea)*0.1
      }     
   }
      
   scale <1,0.34,1>
}     


// *** water object ***
 // isosurface water
 #declare kw=4;
 #declare waterNorth=
 isosurface{
  function{ 
   y
   -f_water(x,0,z)*.05*kw-f_water(x*.5,0,z*.5)*.1*kw-f_water(x*2,0,z*2)*.025*kw-f_water(x*4,0,z*4)*.0125*kw
  }
  max_gradient 1
  contained_by{box{<-SeaSize*.5,-Water_Height,5.0001>,<SeaSize*.5,0.2*kw,SeaSize>}}
  hollow
}

 #declare waterSouth=
 isosurface{
  function{ 
   y
   -f_water(x,0,z)*.05*kw-f_water(x*.5,0,z*.5)*.1*kw-f_water(x*2,0,z*2)*.025*kw-f_water(x*4,0,z*4)*.0125*kw
  }
  max_gradient 1
  contained_by{box{<-10,-Water_Height,-5>,<10,0.2*kw,5>}}
  hollow
}

object { waterNorth 
  
   material { Water_SeanNorth }
   scale <1,1/4,1>
   translate <0,0.2,0>
   
   #if (SW_PHOT)
      photons { collect off }
   #end
}

object { waterSouth 

   material { Water_SeanSouth }
   scale <1,1/4,1>
   translate <0,0.2,0>
   
   #if (SW_PHOT)
      photons {
         target
         reflection on
         refraction on
         collect off
      }
   #end
}

/**************************************************************************************************
  Palms
  
  I used the original version of TOMTREE by Tom Aust:
  
  http://web.archive.org/web/20010306184912/http://www.aust-manufaktur.de/austv2x.html
  
  There's an advanced version by Genady Obukhov:
  http://web.archive.org/web/20071101052625/propro.ru/go/Wshop/povtree/download.html
  
  Both availably via the wayback machine only.
                                              
  I have forgotten where I found the parametrisations for the palms. Most likely from Jaime Vives Piqueres
  (http://www.ignorancia.org/index.php/galleries/old-images/spanish-patio/)  
  and Marc Jaquier ("Fragil", IRTC October 2002, http://ftp.uni-erlangen.de/pub/graphics/irtc/stills/2002-10-31/mj_atoll.zip).
  
  To avoid copyright issues I replaced the single XFROG-Palm (SyagrusRomanzoffiana_Kokospalme_QueenPalm_) here by one of the
  others.

  TOMTREE.inc, the parameter files and some auxiliaries can be found in the archive.
  
***************************************************************************************************/

#macro DisplayValue(Text,Val)
   #debug concat (Text,": ",str(Val.x,8,6),"\n")
#end


#if (SW_Palms)
   #declare PalmsSeed=seed(94+951794);
   #declare PalmsCenter=<12.5,100,-20>;
   #declare PalmsRadius=250;
   #declare PalmsMinLevel=0.6;
   #declare PalmsMaxNumber=300000;
   #declare PalmsNumberPlanted=0;
   #include "palme01.inc"
   #include "palme02.inc"
   #include "palme03.inc"
   #include "palme04.inc"
   #include "palme05.inc"
//   #include "SyagrusRomanzoffiana_Kokospalme_QueenPalm_POV_geom.inc"
   #declare Trees = array [6]
   #declare Trees[0] = object { Palm05 scale 5 }
   #declare Trees[1] = object { Palm01 scale 5 }
   #declare Trees[2] = object { Palm02 scale 5 }
   #declare Trees[3] = object { Palm03 scale 5 }
   #declare Trees[4] = object { Palm04 scale 5 }
   #declare Trees[5] = object { Palm01 scale 5 }//SyagrusRomanzoffiana_Kokospalme_QueenPalm_ rotate <-90,0,0> }//scale 0.5 }
/*   #declare Tuft[6] = object { #include ".inc" scale 1.5 }
   #declare Tuft[7] = object { #include ".inc" scale 1.5 }
   #declare Tuft[8] = object { #include ".inc" scale 1.5 }
   #declare Tuft[9] = object { #include ".inc" scale 1.5 }*/

   #declare Palms = union {
      #declare Count=0;
      #while ( Count < PalmsMaxNumber )
         #declare PalmsNorm = <0,0,0>;
         #declare PalmsRand = rand(PalmsSeed)*PalmsRadius;
         #declare PalmsPos = PalmsCenter*PalmsRand;
         #declare PalmsPos = vaxis_rotate(PalmsPos,y,rand(PalmsSeed)*360);
         #declare PalmsPos = trace (Beach,
                                    PalmsPos,
                                    -y,PalmsNorm);
         #if ( 0 < vlength(PalmsNorm)  & PalmsMinLevel < PalmsPos.y )
            object { Trees[int(rand(PalmsSeed)*5.99)]
          
               scale <1,1.2*rand(PalmsSeed)*0.8,1>  
               rotate <0,rand(PalmsSeed)*360,0>
          
               translate PalmsPos
            }
            #declare PalmsNumberPlanted=PalmsNumberPlanted+1;

         #end
   
         #declare Count = Count + 1;
      #end
   } 

   object { Palms }
#end
#DisplayValue("Palms planted",PalmsNumberPlanted)


/**************************************************************************************************
   Grass
   
   Some years ago i arranged the grass blades by Gilles Tran (http://www.oyonale.com/modeles.php?lang=en&page=2)
   into 10 tufts, since it looks more realistic than single blades. Not really necessary in the backgroud.
   
**************************************************************************************************/



#if (SW_Grass)
   #declare GrassSeed=seed(frame_number+951794);
   #declare GrassCenter=<12.5,100,-20>;
   #declare GrassRadius=250;
   #declare GrassMinLevel=0.5;
   #declare GrassMaxNumber=3000000;
   #declare GrassNumberPlanted=0;
   #include "grass_textures.inc"
   #declare Tuft = array [10]
   #declare Tuft[0] = object { #include "fgrass10.inc" scale 1.5 }
   #declare Tuft[1] = object { #include "fgrass01.inc" scale 1.5 }
   #declare Tuft[2] = object { #include "fgrass02.inc" scale 1.5 }
   #declare Tuft[3] = object { #include "fgrass03.inc" scale 1.5 }
   #declare Tuft[4] = object { #include "fgrass04.inc" scale 1.5 }
   #declare Tuft[5] = object { #include "fgrass05.inc" scale 1.5 }
   #declare Tuft[6] = object { #include "fgrass06.inc" scale 1.5 }
   #declare Tuft[7] = object { #include "fgrass07.inc" scale 1.5 }
   #declare Tuft[8] = object { #include "fgrass08.inc" scale 1.5 }
   #declare Tuft[9] = object { #include "fgrass09.inc" scale 1.5 }

   #declare Grass = union {
      #declare Count=0;
      #while ( Count < GrassMaxNumber )
         #declare GrassNorm = <0,0,0>;
         #declare GrassRand = rand(GrassSeed)*GrassRadius;
         #declare GrassPos = GrassCenter*GrassRand;
         #declare GrassPos = vaxis_rotate(GrassPos,y,rand(GrassSeed)*360);
         #declare GrassPos = trace (Beach,
                                    GrassPos,
                                    -y,GrassNorm);
         #if ( 0 < vlength(GrassNorm)  & GrassMinLevel < GrassPos.y )
            object { Tuft[int(rand(GrassSeed)*9.99)]
          
               texture { T_Grass01 scale 30 }
                 
               rotate <0,rand(GrassSeed)*360,0>
//               scale 0.004
               scale 0.01          
          
               translate GrassPos
            }
            #declare GrassNumberPlanted=GrassNumberPlanted+1;

         #end
   
         #declare Count = Count + 1;
      #end
   } 

   object { Grass }
#end
#DisplayValue("Grass tufts planted",GrassNumberPlanted)


/**************************************************************************************************
   Dugout trimaran
   
   Finally, I modelled this boats with Wings 3D (http://www.wings3d.com/) after screenshots
   of a video documentation about Zanzibar. 
**************************************************************************************************/


#include "texturen.inc"
#include "Trimaran12_POV_geom.inc"
#include "Trimaran12_MastLiegend1_POV_geom.inc"
#include "Trimaran12_MastLiegend2_POV_geom.inc"
#include "Trimaran12_MastAufrecht_POV_geom.inc"

#declare TrimaranMastLL=union {
   object { Trimaran12_ }
   object { Trimaran12_MastLiegend1_ }
   object { Trimaran12_MastLiegend2_ }
}
object { TrimaranMastLL
   scale 1/4*<-1,1,-1>
   rotate <0,0,3>
   rotate <0,45,0>
   translate <1,0.15,-2>
   #if (SW_PHOT)
      photons { collect on }
   #end
}

#declare TrimaranMastA=union {
   object { Trimaran12_ }
   object { Trimaran12_MastAufrecht_ }
}

object { TrimaranMastA
   scale 1/4*<1,1,-1>
   rotate <0,0,3>
   rotate <0,80,0>
   translate <0,0.15,2.8> // weiter weg!
   #if (SW_PHOT)
      photons { collect on }
   #end
}


#declare BoatSeed=seed(83+577901);
#for(ix,1,5)
   object { TrimaranMastA
      scale 1/4*<1,1,-1>
      rotate <0,0,-4+rand(BoatSeed)*8>
      rotate <0,35+rand(BoatSeed)*20,0>
      translate <5,0.15,30>+<-10+rand(BoatSeed)*20,0,-10+rand(BoatSeed)*20>
      #if (SW_PHOT)
         photons { collect off }
      #end
   }
#end
#for(ix,1,5)
   object { TrimaranMastLL
      scale 1/4*<1,1,-1>
      rotate <0,0,-4+rand(BoatSeed)*8>
      rotate <0,35+rand(BoatSeed)*20,0>
      translate <5,0.15,40>+<-10+rand(BoatSeed)*20,0,-10+rand(BoatSeed)*20>
      #if (SW_PHOT)
         photons { collect off }
      #end
   }
#end
